@@ -18,12 +18,14 @@ from python_kemptech_api.exceptions import (
     RealServerMissingVirtualServiceInfo,
     LoadMasterParameterError,
     ValidationError,
+    SubVsCannotCreateSubVs,
 )
 from .utils import (
     validate_port,
     validate_ip,
     validate_protocol,
     UseTlsAdapter,
+    get_sub_vs_list_from_data,
 )
 
 
@@ -398,19 +400,32 @@ class KempBaseObjectModel(object):
 
 class VirtualService(HttpClient, KempBaseObjectModel):
 
-    def __init__(self, loadmaster_info, ip, port=80, prot="tcp",
+    def __init__(self, loadmaster_info, ip_or_index, port=80, prot="tcp",
                  is_sub_vs=False):
+        """Construct VirtualService object.
+
+        :param loadmaster_info: The loadmaster dict with the endpoint params.
+        :param ip_or_index: IP or index of the VS. When creating a subvs you
+               must pass the index and set the is_sub_vs flag to true in order
+               for createsubvs to behave correctly. The index will be
+               overwritten with the index of the newly created subvs on save().
+        :param port: Port of the virtual service.
+        :param prot: Protocol of the virtual service.
+        :param is_sub_vs: Whether or not it is a subvs, mark this as true and
+               pass the parent VS index as the ip_or_index parameter.
+        """
         self.index = None  # to avoid AttributeErrors later
+        self._is_sub_vs = is_sub_vs
         if not is_sub_vs:
             # Skip validation when it is a subvs as they do not have ip/port
-            self.vs = ip
-            validate_ip(ip)
-
+            self.vs = ip_or_index
             self.port = port
+            self.prot = prot
+            validate_ip(ip_or_index)
             validate_port(port)
-
-        self.prot = prot
-        validate_protocol(prot)
+            validate_protocol(prot)
+        else:
+            self.index = ip_or_index
 
         try:
             self.endpoint = loadmaster_info["endpoint"]
@@ -423,8 +438,13 @@ class VirtualService(HttpClient, KempBaseObjectModel):
         super(VirtualService, self).__init__()
 
     def __str__(self):
-        return 'Virtual Service {} {}:{} on LoadMaster {}'.format(
-            self.prot.upper(), self.vs, self.port, self.ip_address)
+        try:
+            if int(self.vs):
+                return 'Sub Virtual Service {} on LoadMaster {}'.format(
+                    self.vs, self.ip_address)
+        except ValueError:
+            return 'Virtual Service {} {}:{} on LoadMaster {}'.format(
+                self.prot.upper(), self.vs, self.port, self.ip_address)
 
     def _get_base_parameters(self):
         """Returns the bare minimum VS parameters. IP, port and protocol"""
@@ -452,9 +472,25 @@ class VirtualService(HttpClient, KempBaseObjectModel):
 
     def save(self, update=False):
         if not update:
-            response = self._get("/addvs", self.to_api_dict())
+            if self._is_sub_vs:
+                # Hell, thy name be subvs
+                response = self._get("/showvs", self._get_base_parameters())
+                data = get_data(response)
+                existing_subvs_entries = get_sub_vs_list_from_data(data)
+                params = self._get_base_parameters()
+                params["createsubvs"] = ""
+                response = self._get("/modvs", params)
+                data = get_data(response)
+                new_subvs_entries = get_sub_vs_list_from_data(data)
+                s = set(existing_subvs_entries)
+                created_subvs_id = [x for x in new_subvs_entries if x not in s]
+                newly_created_vs_params = {"vs": created_subvs_id}
+                response = self._get("/showvs", newly_created_vs_params)
+            else:
+                response = self._get("/addvs", self.to_api_dict())
         else:
             response = self._get("/modvs", self.to_api_dict())
+
         if is_successful(response):
             vs_data = get_data(response)
             self.populate_default_attributes(vs_data)
@@ -484,6 +520,19 @@ class VirtualService(HttpClient, KempBaseObjectModel):
         self.qos = service.get('QoS', None)
         self.idletime = service.get('Idletime', None)
 
+    def create_sub_virtual_service(self):
+        """Create a virtual service marked as subvs
+
+        When creating a virtual service that is a sub virtual service you must
+        pass the parent index to the constructor and mark the is_sub_vs flag
+        as true. This will allow the save() method on the newly created subvs
+        instance to be able to create a subvs against the parent vs. The index
+        attribute will then be overwritten on save with the subvs's index.
+        """
+        if self._is_sub_vs:
+            raise SubVsCannotCreateSubVs()
+        return VirtualService(self.access_info, self.index, is_sub_vs=True)
+
     def create_real_server(self, ip, port=80):
         """RealServer factory with pre-configured LoadMaster connection."""
         return RealServer(self.access_info, ip, port)
