@@ -45,3 +45,15 @@ class UseTlsAdapter(HTTPAdapter):
         self.poolmanager = PoolManager(num_pools=connections,
                                        maxsize=maxsize, block=block,
                                        ssl_version=TLS_VERSION)
+
+
+def get_sub_vs_list_from_data(response_data):
+    subvs_entries = []
+    for key, value in response_data.items():
+        if key == "SubVS":
+            if isinstance(value, list):
+                for subvs in value:
+                    subvs_entries.append(subvs['VSIndex'])
+            else:
+                subvs_entries.append(value['VSIndex'])
+    return subvs_entries
