@@ -53,6 +53,13 @@ class Test_VirtualService:
         }
         assert_equal(base_params, expected_params)
 
+        self.vs.index = 1
+        base_params = self.vs._get_base_parameters()
+        expected_params = {
+            "vs": 1,
+        }
+        assert_equal(base_params, expected_params)
+
     def test_to_api_dict(self):
         actual = self.vs.to_api_dict()
         expected = {
@@ -62,6 +69,18 @@ class Test_VirtualService:
         }
         assert_equal(actual, expected)
 
+    def test_to_dict(self):
+        self.vs._ignore = None
+        actual = self.vs.to_dict()
+        expected = {
+            "endpoint": "https://bal:2fourall@1.1.1.1:443/access",
+            "ip_address": "1.1.1.1",
+            "vs": "1.1.1.2",
+            "port": 80,
+            "prot": "tcp",
+        }
+        assert_equal(actual, expected)
+
 class Test_get_real_servers:
 
     def setup(self):
